@@ -19,10 +19,9 @@
 
 import time
 from collections import OrderedDict
-from datetime import datetime
 from typing import Optional, Set
 
-from sqlalchemy import and_
+import pendulum
 from sqlalchemy.orm.session import Session, make_transient
 from tabulate import tabulate
 
@@ -42,8 +41,9 @@ from airflow.models.dagrun import DagRun
 from airflow.models.taskinstance import TaskInstance, TaskInstanceKey
 from airflow.ti_deps.dep_context import DepContext
 from airflow.ti_deps.dependencies_deps import BACKFILL_QUEUED_DEPS
+from airflow.timetables.base import DagRunInfo
 from airflow.utils import helpers, timezone
-from airflow.utils.configuration import tmp_configuration_copy
+from airflow.utils.configuration import conf as airflow_conf, tmp_configuration_copy
 from airflow.utils.session import provide_session
 from airflow.utils.state import State
 from airflow.utils.types import DagRunType
@@ -95,7 +95,7 @@ class BackfillJob(BaseJob):
         """
 
         # TODO(edgarRd): AIRFLOW-1444: Add consistency check on counts
-        def __init__(  # pylint: disable=too-many-arguments
+        def __init__(
             self,
             to_run=None,
             running=None,
@@ -121,7 +121,7 @@ class BackfillJob(BaseJob):
             self.finished_runs = finished_runs
             self.total_runs = total_runs
 
-    def __init__(  # pylint: disable=too-many-arguments
+    def __init__(
         self,
         dag,
         start_date=None,
@@ -136,6 +136,7 @@ class BackfillJob(BaseJob):
         conf=None,
         rerun_failed_tasks=False,
         run_backwards=False,
+        run_at_least_once=False,
         *args,
         **kwargs,
     ):
@@ -166,6 +167,9 @@ class BackfillJob(BaseJob):
         :type rerun_failed_tasks: bool
         :param run_backwards: Whether to process the dates from most to least recent
         :type run_backwards bool
+        :param run_at_least_once: If true, always run the DAG at least once even
+            if no logical run exists within the time range.
+        :type: bool
         :param args:
         :param kwargs:
         """
@@ -183,6 +187,7 @@ class BackfillJob(BaseJob):
         self.conf = conf
         self.rerun_failed_tasks = rerun_failed_tasks
         self.run_backwards = run_backwards
+        self.run_at_least_once = run_at_least_once
         super().__init__(*args, **kwargs)
 
     @provide_session
@@ -228,7 +233,9 @@ class BackfillJob(BaseJob):
             # special case: if the task needs to be rescheduled put it back
             elif ti.state == State.UP_FOR_RESCHEDULE:
                 self.log.warning("Task instance %s is up for reschedule", ti)
-                ti_status.running.pop(reduced_key)
+                # During handling of reschedule state in ti._handle_reschedule, try number is reduced
+                # by one, so we should not use reduced_key to avoid key error
+                ti_status.running.pop(ti.key)
                 ti_status.to_run[ti.key] = ti
             # special case: The state of the task can be set to NONE by the task itself
             # when it reaches concurrency limits. It could also happen when the state
@@ -283,19 +290,21 @@ class BackfillJob(BaseJob):
                 ti.handle_failure_with_callback(error=msg)
 
     @provide_session
-    def _get_dag_run(self, run_date: datetime, dag: DAG, session: Session = None):
+    def _get_dag_run(self, dagrun_info: DagRunInfo, dag: DAG, session: Session = None):
         """
         Returns a dag run for the given run date, which will be matched to an existing
         dag run if available or create a new dag run otherwise. If the max_active_runs
         limit is reached, this function will return None.
 
-        :param run_date: the execution date for the dag run
+        :param dagrun_info: Schedule information for the dag run
         :param dag: DAG
         :param session: the database session object
         :return: a DagRun in state RUNNING or None
         """
+        run_date = dagrun_info.logical_date
+
         # consider max_active_runs but ignore when running subdags
-        respect_dag_max_active_limit = bool(dag.schedule_interval and not dag.is_subdag)
+        respect_dag_max_active_limit = bool(dag.timetable.can_run and not dag.is_subdag)
 
         current_active_dag_count = dag.get_num_active_runs(external_trigger=False)
 
@@ -317,6 +326,7 @@ class BackfillJob(BaseJob):
 
         run = run or dag.create_dagrun(
             execution_date=run_date,
+            data_interval=dagrun_info.data_interval,
             start_date=timezone.utcnow(),
             state=State.RUNNING,
             external_trigger=False,
@@ -331,7 +341,6 @@ class BackfillJob(BaseJob):
 
         # explicitly mark as backfill and running
         run.state = State.RUNNING
-        run.run_id = run.generate_run_id(DagRunType.BACKFILL_JOB, run_date)
         run.run_type = DagRunType.BACKFILL_JOB
         run.verify_integrity(session=session)
         return run
@@ -391,7 +400,7 @@ class BackfillJob(BaseJob):
         self.log.debug("Finished dag run loop iteration. Remaining tasks %s", ti_status.to_run.values())
 
     @provide_session
-    def _process_backfill_task_instances(  # pylint: disable=too-many-statements
+    def _process_backfill_task_instances(
         self,
         ti_status,
         executor,
@@ -419,6 +428,8 @@ class BackfillJob(BaseJob):
         """
         executed_run_dates = []
 
+        is_unit_test = airflow_conf.getboolean('core', 'unit_test_mode')
+
         while (len(ti_status.to_run) > 0 or len(ti_status.running) > 0) and len(ti_status.deadlocked) == 0:
             self.log.debug("*** Clearing out not_ready list ***")
             ti_status.not_ready.clear()
@@ -428,15 +439,12 @@ class BackfillJob(BaseJob):
             # determined deadlocked while they are actually
             # waiting for their upstream to finish
             @provide_session
-            def _per_task_process(key, ti, session=None):  # pylint: disable=too-many-return-statements
+            def _per_task_process(key, ti: TaskInstance, session=None):
                 ti.refresh_from_db(lock_for_update=True, session=session)
 
                 task = self.dag.get_task(ti.task_id, include_subdags=True)
                 ti.task = task
 
-                ignore_depends_on_past = self.ignore_first_depends_on_past and ti.execution_date == (
-                    start_date or ti.start_date
-                )
                 self.log.debug("Task instance to run %s state %s", ti, ti.state)
 
                 # The task was already marked successful or skipped by a
@@ -460,7 +468,7 @@ class BackfillJob(BaseJob):
                 # in case max concurrency has been reached at task runtime
                 elif ti.state == State.NONE:
                     self.log.warning(
-                        "FIXME: task instance {} state was set to None " "externally. This should not happen"
+                        "FIXME: Task instance %s state was set to None externally. This should not happen", ti
                     )
                     ti.set_state(State.SCHEDULED, session=session)
                 if self.rerun_failed_tasks:
@@ -481,6 +489,12 @@ class BackfillJob(BaseJob):
                             ti_status.running.pop(key)
                         return
 
+                if self.ignore_first_depends_on_past:
+                    dagrun = ti.get_dagrun(session=session)
+                    ignore_depends_on_past = dagrun.execution_date == (start_date or ti.start_date)
+                else:
+                    ignore_depends_on_past = False
+
                 backfill_context = DepContext(
                     deps=BACKFILL_QUEUED_DEPS,
                     ignore_depends_on_past=ignore_depends_on_past,
@@ -494,9 +508,7 @@ class BackfillJob(BaseJob):
                     dep_context=backfill_context, session=session, verbose=self.verbose
                 ):
                     if executor.has_task(ti):
-                        self.log.debug(
-                            "Task Instance %s already in executor " "waiting for queue to clear", ti
-                        )
+                        self.log.debug("Task Instance %s already in executor waiting for queue to clear", ti)
                     else:
                         self.log.debug('Sending %s to executor', ti)
                         # Skip scheduled state, we are executing immediately
@@ -536,7 +548,7 @@ class BackfillJob(BaseJob):
 
                 # special case
                 if ti.state == State.UP_FOR_RETRY:
-                    self.log.debug("Task instance %s retry period not " "expired yet", ti)
+                    self.log.debug("Task instance %s retry period not expired yet", ti)
                     if key in ti_status.running:
                         ti_status.running.pop(key)
                     ti_status.to_run[key] = ti
@@ -544,7 +556,7 @@ class BackfillJob(BaseJob):
 
                 # special case
                 if ti.state == State.UP_FOR_RESCHEDULE:
-                    self.log.debug("Task instance %s reschedule period not " "expired yet", ti)
+                    self.log.debug("Task instance %s reschedule period not expired yet", ti)
                     if key in ti_status.running:
                         ti_status.running.pop(key)
                     ti_status.to_run[key] = ti
@@ -554,7 +566,7 @@ class BackfillJob(BaseJob):
                 self.log.debug('Adding %s to not_ready', ti)
                 ti_status.not_ready.add(key)
 
-            try:  # pylint: disable=too-many-nested-blocks
+            try:
                 for task in self.dag.topological_sort(include_subdag_tasks=True):
                     for key, ti in list(ti_status.to_run.items()):
                         if task.task_id != ti.task_id:
@@ -574,31 +586,33 @@ class BackfillJob(BaseJob):
                         num_running_task_instances_in_dag = DAG.get_num_task_instances(
                             self.dag_id,
                             states=self.STATES_COUNT_AS_RUNNING,
+                            session=session,
                         )
 
-                        if num_running_task_instances_in_dag >= self.dag.concurrency:
+                        if num_running_task_instances_in_dag >= self.dag.max_active_tasks:
                             raise DagConcurrencyLimitReached(
-                                "Not scheduling since DAG concurrency limit " "is reached."
+                                "Not scheduling since DAG max_active_tasks limit is reached."
                             )
 
-                        if task.task_concurrency:
+                        if task.max_active_tis_per_dag:
                             num_running_task_instances_in_task = DAG.get_num_task_instances(
                                 dag_id=self.dag_id,
                                 task_ids=[task.task_id],
                                 states=self.STATES_COUNT_AS_RUNNING,
+                                session=session,
                             )
 
-                            if num_running_task_instances_in_task >= task.task_concurrency:
+                            if num_running_task_instances_in_task >= task.max_active_tis_per_dag:
                                 raise TaskConcurrencyLimitReached(
-                                    "Not scheduling since Task concurrency limit " "is reached."
+                                    "Not scheduling since Task concurrency limit is reached."
                                 )
 
                         _per_task_process(key, ti)
             except (NoAvailablePoolSlot, DagConcurrencyLimitReached, TaskConcurrencyLimitReached) as e:
                 self.log.debug(e)
 
+            self.heartbeat(only_if_necessary=is_unit_test)
             # execute the tasks in the queue
-            self.heartbeat()
             executor.heartbeat()
 
             # If the set of tasks that aren't ready ever equals the set of
@@ -639,17 +653,15 @@ class BackfillJob(BaseJob):
             # Sorting by execution date first
             sorted_ti_keys = sorted(
                 set_ti_keys,
-                key=lambda ti_key: (ti_key.execution_date, ti_key.dag_id, ti_key.task_id, ti_key.try_number),
+                key=lambda ti_key: (ti_key.run_id, ti_key.dag_id, ti_key.task_id, ti_key.try_number),
             )
-            return tabulate(sorted_ti_keys, headers=["DAG ID", "Task ID", "Execution date", "Try number"])
+            return tabulate(sorted_ti_keys, headers=["DAG ID", "Task ID", "Run ID", "Try number"])
 
         def tabulate_tis_set(set_tis: Set[TaskInstance]) -> str:
             # Sorting by execution date first
-            sorted_tis = sorted(
-                set_tis, key=lambda ti: (ti.execution_date, ti.dag_id, ti.task_id, ti.try_number)
-            )
-            tis_values = ((ti.dag_id, ti.task_id, ti.execution_date, ti.try_number) for ti in sorted_tis)
-            return tabulate(tis_values, headers=["DAG ID", "Task ID", "Execution date", "Try number"])
+            sorted_tis = sorted(set_tis, key=lambda ti: (ti.run_id, ti.dag_id, ti.task_id, ti.try_number))
+            tis_values = ((ti.dag_id, ti.task_id, ti.run_id, ti.try_number) for ti in sorted_tis)
+            return tabulate(tis_values, headers=["DAG ID", "Task ID", "Run ID", "Try number"])
 
         err = ''
         if ti_status.failed:
@@ -690,14 +702,14 @@ class BackfillJob(BaseJob):
         return err
 
     @provide_session
-    def _execute_for_run_dates(self, run_dates, ti_status, executor, pickle_id, start_date, session=None):
+    def _execute_dagruns(self, dagrun_infos, ti_status, executor, pickle_id, start_date, session=None):
         """
         Computes the dag runs and their respective task instances for
         the given run dates and executes the task instances.
         Returns a list of execution dates of the dag runs that were executed.
 
-        :param run_dates: Execution dates for dag runs
-        :type run_dates: list
+        :param dagrun_infos: Schedule information for dag runs
+        :type dagrun_infos: list[DagRunInfo]
         :param ti_status: internal BackfillJob status structure to tis track progress
         :type ti_status: BackfillJob._DagRunTaskStatus
         :param executor: the executor to use, it must be previously started
@@ -709,9 +721,9 @@ class BackfillJob(BaseJob):
         :param session: the current session object
         :type session: sqlalchemy.orm.session.Session
         """
-        for next_run_date in run_dates:
+        for dagrun_info in dagrun_infos:
             for dag in [self.dag] + self.dag.subdags:
-                dag_run = self._get_dag_run(next_run_date, dag, session=session)
+                dag_run = self._get_dag_run(dagrun_info, dag, session=session)
                 tis_map = self._task_instances_for_dag_run(dag_run, session=session)
                 if dag_run is None:
                     continue
@@ -755,8 +767,13 @@ class BackfillJob(BaseJob):
 
         start_date = self.bf_start_date
 
-        # Get intervals between the start/end dates, which will turn into dag runs
-        run_dates = self.dag.get_run_dates(start_date=start_date, end_date=self.bf_end_date)
+        # Get DagRun schedule between the start/end dates, which will turn into dag runs.
+        dagrun_start_date = timezone.coerce_datetime(start_date)
+        if self.bf_end_date is None:
+            dagrun_end_date = pendulum.now(timezone.utc)
+        else:
+            dagrun_end_date = pendulum.instance(self.bf_end_date)
+        dagrun_infos = list(self.dag.iter_dagrun_infos_between(dagrun_start_date, dagrun_end_date))
         if self.run_backwards:
             tasks_that_depend_on_past = [t.task_id for t in self.dag.task_dict.values() if t.depends_on_past]
             if tasks_that_depend_on_past:
@@ -765,11 +782,13 @@ class BackfillJob(BaseJob):
                         ",".join(tasks_that_depend_on_past)
                     )
                 )
-            run_dates = run_dates[::-1]
+            dagrun_infos = dagrun_infos[::-1]
 
-        if len(run_dates) == 0:
-            self.log.info("No run dates were found for the given dates and dag interval.")
-            return
+        if not dagrun_infos:
+            if not self.run_at_least_once:
+                self.log.info("No run dates were found for the given dates and dag interval.")
+                return
+            dagrun_infos = [DagRunInfo.interval(dagrun_start_date, dagrun_end_date)]
 
         # picklin'
         pickle_id = None
@@ -788,17 +807,18 @@ class BackfillJob(BaseJob):
         executor.job_id = "backfill"
         executor.start()
 
-        ti_status.total_runs = len(run_dates)  # total dag runs in backfill
+        ti_status.total_runs = len(dagrun_infos)  # total dag runs in backfill
 
-        try:  # pylint: disable=too-many-nested-blocks
+        try:
             remaining_dates = ti_status.total_runs
             while remaining_dates > 0:
-                dates_to_process = [
-                    run_date for run_date in run_dates if run_date not in ti_status.executed_dag_run_dates
+                dagrun_infos_to_process = [
+                    dagrun_info
+                    for dagrun_info in dagrun_infos
+                    if dagrun_info.logical_date not in ti_status.executed_dag_run_dates
                 ]
-
-                self._execute_for_run_dates(
-                    run_dates=dates_to_process,
+                self._execute_dagruns(
+                    dagrun_infos=dagrun_infos_to_process,
                     ti_status=ti_status,
                     executor=executor,
                     pickle_id=pickle_id,
@@ -848,19 +868,13 @@ class BackfillJob(BaseJob):
         # also consider running as the state might not have changed in the db yet
         running_tis = self.executor.running
 
+        # Can't use an update here since it doesn't support joins.
         resettable_states = [State.SCHEDULED, State.QUEUED]
         if filter_by_dag_run is None:
             resettable_tis = (
                 session.query(TaskInstance)
-                .join(
-                    DagRun,
-                    and_(
-                        TaskInstance.dag_id == DagRun.dag_id,
-                        TaskInstance.execution_date == DagRun.execution_date,
-                    ),
-                )
+                .join(TaskInstance.dag_run)
                 .filter(
-                    # pylint: disable=comparison-with-callable
                     DagRun.state == State.RUNNING,
                     DagRun.run_type != DagRunType.BACKFILL_JOB,
                     TaskInstance.state.in_(resettable_states),
@@ -868,12 +882,8 @@ class BackfillJob(BaseJob):
             ).all()
         else:
             resettable_tis = filter_by_dag_run.get_task_instances(state=resettable_states, session=session)
-        tis_to_reset = []
-        # Can't use an update here since it doesn't support joins
-        for ti in resettable_tis:
-            if ti.key not in queued_tis and ti.key not in running_tis:
-                tis_to_reset.append(ti)
 
+        tis_to_reset = [ti for ti in resettable_tis if ti.key not in queued_tis and ti.key not in running_tis]
         if not tis_to_reset:
             return 0
 
@@ -897,8 +907,8 @@ class BackfillJob(BaseJob):
 
         reset_tis = helpers.reduce_in_chunks(query, tis_to_reset, [], self.max_tis_per_query)
 
-        task_instance_str = '\n\t'.join([repr(x) for x in reset_tis])
-        session.commit()
+        task_instance_str = '\n\t'.join(repr(x) for x in reset_tis)
+        session.flush()
 
         self.log.info("Reset the following %s TaskInstances:\n\t%s", len(reset_tis), task_instance_str)
         return len(reset_tis)
