@@ -21,32 +21,29 @@ import enum
 import logging
 from dataclasses import dataclass
 from inspect import Parameter, signature
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Set, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Set, Type, Union
 
 import cattr
 import pendulum
 from dateutil import relativedelta
+from pendulum.tz.timezone import FixedTimezone, Timezone
 
-try:
-    from functools import cache
-except ImportError:
-    from functools import lru_cache
-
-    cache = lru_cache(maxsize=None)
-from pendulum.tz.timezone import Timezone
-
+from airflow.compat.functools import cache
 from airflow.configuration import conf
 from airflow.exceptions import AirflowException, SerializationError
 from airflow.models.baseoperator import BaseOperator, BaseOperatorLink
 from airflow.models.connection import Connection
-from airflow.models.dag import DAG
+from airflow.models.dag import DAG, create_timetable
+from airflow.models.param import Param, ParamsDict
 from airflow.providers_manager import ProvidersManager
 from airflow.serialization.enums import DagAttributeTypes as DAT, Encoding
 from airflow.serialization.helpers import serialize_template_field
 from airflow.serialization.json_schema import Validator, load_dag_schema
 from airflow.settings import json
+from airflow.timetables.base import Timetable
 from airflow.utils.code_utils import get_python_source
-from airflow.utils.module_loading import import_string
+from airflow.utils.docs import get_docs_url
+from airflow.utils.module_loading import as_importable_string, import_string
 from airflow.utils.task_group import TaskGroup
 
 try:
@@ -59,11 +56,9 @@ try:
 except ImportError:
     HAS_KUBERNETES = False
 
-
 if TYPE_CHECKING:
     from airflow.ti_deps.deps.base_ti_dep import BaseTIDep
 
-
 log = logging.getLogger(__name__)
 
 _OPERATOR_EXTRA_LINKS: Set[str] = {
@@ -87,6 +82,92 @@ def get_operator_extra_links():
     return _OPERATOR_EXTRA_LINKS
 
 
+def encode_relativedelta(var: relativedelta.relativedelta) -> Dict[str, Any]:
+    encoded = {k: v for k, v in var.__dict__.items() if not k.startswith("_") and v}
+    if var.weekday and var.weekday.n:
+        # Every n'th Friday for example
+        encoded['weekday'] = [var.weekday.weekday, var.weekday.n]
+    elif var.weekday:
+        encoded['weekday'] = [var.weekday.weekday]
+    return encoded
+
+
+def decode_relativedelta(var: Dict[str, Any]) -> relativedelta.relativedelta:
+    if 'weekday' in var:
+        var['weekday'] = relativedelta.weekday(*var['weekday'])  # type: ignore
+    return relativedelta.relativedelta(**var)
+
+
+def encode_timezone(var: Timezone) -> Union[str, int]:
+    """Encode a Pendulum Timezone for serialization.
+
+    Airflow only supports timezone objects that implements Pendulum's Timezone
+    interface. We try to keep as much information as possible to make conversion
+    round-tripping possible (see ``decode_timezone``). We need to special-case
+    UTC; Pendulum implements it as a FixedTimezone (i.e. it gets encoded as
+    0 without the special case), but passing 0 into ``pendulum.timezone`` does
+    not give us UTC (but ``+00:00``).
+    """
+    if isinstance(var, FixedTimezone):
+        if var.offset == 0:
+            return "UTC"
+        return var.offset
+    if isinstance(var, Timezone):
+        return var.name
+    raise ValueError(
+        f"DAG timezone should be a pendulum.tz.Timezone, not {var!r}. "
+        f"See {get_docs_url('timezone.html#time-zone-aware-dags')}"
+    )
+
+
+def decode_timezone(var: Union[str, int]) -> Timezone:
+    """Decode a previously serialized Pendulum Timezone."""
+    return pendulum.timezone(var)
+
+
+def _get_registered_timetable(importable_string: str) -> Optional[Type[Timetable]]:
+    from airflow import plugins_manager
+
+    if importable_string.startswith("airflow.timetables."):
+        return import_string(importable_string)
+    plugins_manager.initialize_timetables_plugins()
+    return plugins_manager.timetable_classes.get(importable_string)
+
+
+class _TimetableNotRegistered(ValueError):
+    def __init__(self, type_string: str) -> None:
+        self.type_string = type_string
+
+    def __str__(self) -> str:
+        return f"Timetable class {self.type_string!r} is not registered"
+
+
+def _encode_timetable(var: Timetable) -> Dict[str, Any]:
+    """Encode a timetable instance.
+
+    This delegates most of the serialization work to the type, so the behavior
+    can be completely controlled by a custom subclass.
+    """
+    timetable_class = type(var)
+    importable_string = as_importable_string(timetable_class)
+    if _get_registered_timetable(importable_string) is None:
+        raise _TimetableNotRegistered(importable_string)
+    return {"__type": importable_string, "__var": var.serialize()}
+
+
+def _decode_timetable(var: Dict[str, Any]) -> Timetable:
+    """Decode a previously serialized timetable.
+
+    Most of the deserialization logic is delegated to the actual type, which
+    we import from string.
+    """
+    importable_string = var["__type"]
+    timetable_class = _get_registered_timetable(importable_string)
+    if timetable_class is None:
+        raise _TimetableNotRegistered(importable_string)
+    return timetable_class.deserialize(var["__var"])
+
+
 class BaseSerialization:
     """BaseSerialization provides utils for serialization."""
 
@@ -188,6 +269,8 @@ class BaseSerialization:
 
             if key in decorated_fields:
                 serialized_object[key] = cls._serialize(value)
+            elif key == "timetable":
+                serialized_object[key] = _encode_timetable(value)
             else:
                 value = cls._serialize(value)
                 if isinstance(value, dict) and "__type" in value:
@@ -195,7 +278,6 @@ class BaseSerialization:
                 serialized_object[key] = value
         return serialized_object
 
-    # pylint: disable=too-many-return-statements
     @classmethod
     def _serialize(cls, var: Any) -> Any:  # Unfortunately there is no support for recursive types in mypy
         """Helper function of depth first search for serialization.
@@ -229,15 +311,9 @@ class BaseSerialization:
         elif isinstance(var, datetime.timedelta):
             return cls._encode(var.total_seconds(), type_=DAT.TIMEDELTA)
         elif isinstance(var, Timezone):
-            return cls._encode(str(var.name), type_=DAT.TIMEZONE)
+            return cls._encode(encode_timezone(var), type_=DAT.TIMEZONE)
         elif isinstance(var, relativedelta.relativedelta):
-            encoded = {k: v for k, v in var.__dict__.items() if not k.startswith("_") and v}
-            if var.weekday and var.weekday.n:
-                # Every n'th Friday for example
-                encoded['weekday'] = [var.weekday.weekday, var.weekday.n]
-            elif var.weekday:
-                encoded['weekday'] = [var.weekday.weekday]
-            return cls._encode(encoded, type_=DAT.RELATIVEDELTA)
+            return cls._encode(encode_relativedelta(var), type_=DAT.RELATIVEDELTA)
         elif callable(var):
             return str(get_python_source(var))
         elif isinstance(var, set):
@@ -251,14 +327,14 @@ class BaseSerialization:
             return cls._encode([cls._serialize(v) for v in var], type_=DAT.TUPLE)
         elif isinstance(var, TaskGroup):
             return SerializedTaskGroup.serialize_task_group(var)
+        elif isinstance(var, Param):
+            return cls._encode(cls._serialize_param(var), type_=DAT.PARAM)
         else:
             log.debug('Cast type %s to str in serialization.', type(var))
             return str(var)
 
-    # pylint: enable=too-many-return-statements
-
     @classmethod
-    def _deserialize(cls, encoded_var: Any) -> Any:  # pylint: disable=too-many-return-statements
+    def _deserialize(cls, encoded_var: Any) -> Any:
         """Helper function of depth first search for deserialization."""
         # JSON primitives (except for dict) are not encoded.
         if cls._is_primitive(encoded_var):
@@ -287,15 +363,15 @@ class BaseSerialization:
         elif type_ == DAT.TIMEDELTA:
             return datetime.timedelta(seconds=var)
         elif type_ == DAT.TIMEZONE:
-            return Timezone(var)
+            return decode_timezone(var)
         elif type_ == DAT.RELATIVEDELTA:
-            if 'weekday' in var:
-                var['weekday'] = relativedelta.weekday(*var['weekday'])  # type: ignore
-            return relativedelta.relativedelta(**var)
+            return decode_relativedelta(var)
         elif type_ == DAT.SET:
             return {cls._deserialize(v) for v in var}
         elif type_ == DAT.TUPLE:
             return tuple(cls._deserialize(v) for v in var)
+        elif type_ == DAT.PARAM:
+            return cls._deserialize_param(var)
         else:
             raise TypeError(f'Invalid type {type_!s} in deserialization.')
 
@@ -308,7 +384,7 @@ class BaseSerialization:
 
     @classmethod
     def _is_constructor_param(cls, attrname: str, instance: Any) -> bool:
-        # pylint: disable=unused-argument
+
         return attrname in cls._CONSTRUCTOR_PARAMS
 
     @classmethod
@@ -316,8 +392,8 @@ class BaseSerialization:
         """
         Return true if ``value`` is the hard-coded default for the given attribute.
 
-        This takes in to account cases where the ``concurrency`` parameter is
-        stored in the ``_concurrency`` attribute.
+        This takes in to account cases where the ``max_active_tasks`` parameter is
+        stored in the ``_max_active_tasks`` attribute.
 
         And by using `is` here only and not `==` this copes with the case a
         user explicitly specifies an attribute with the same "value" as the
@@ -328,13 +404,69 @@ class BaseSerialization:
         to account for the case where the default value of the field is None but has the
         ``field = field or {}`` set.
         """
-        # pylint: disable=unused-argument
         if attrname in cls._CONSTRUCTOR_PARAMS and (
             cls._CONSTRUCTOR_PARAMS[attrname] is value or (value in [{}, []])
         ):
             return True
         return False
 
+    @classmethod
+    def _serialize_param(cls, param: Param):
+        return dict(
+            __class=f"{param.__module__}.{param.__class__.__name__}",
+            default=cls._serialize(param.value),
+            description=cls._serialize(param.description),
+            schema=cls._serialize(param.schema),
+        )
+
+    @classmethod
+    def _deserialize_param(cls, param_dict: Dict):
+        """
+        In 2.2.0, Param attrs were assumed to be json-serializable and were not run through
+        this class's ``_serialize`` method.  So before running through ``_deserialize``,
+        we first verify that it's necessary to do.
+        """
+        class_name = param_dict['__class']
+        class_ = import_string(class_name)  # type: Type[Param]
+        attrs = ('default', 'description', 'schema')
+        kwargs = {}
+        for attr in attrs:
+            if attr not in param_dict:
+                continue
+            val = param_dict[attr]
+            is_serialized = isinstance(val, dict) and '__type' in val
+            if is_serialized:
+                deserialized_val = cls._deserialize(param_dict[attr])
+                kwargs[attr] = deserialized_val
+            else:
+                kwargs[attr] = val
+        return class_(**kwargs)
+
+    @classmethod
+    def _serialize_params_dict(cls, params: ParamsDict):
+        """Serialize Params dict for a DAG/Task"""
+        serialized_params = {}
+        for k, v in params.items():
+            # TODO: As of now, we would allow serialization of params which are of type Param only
+            if f'{v.__module__}.{v.__class__.__name__}' == 'airflow.models.param.Param':
+                serialized_params[k] = cls._serialize_param(v)
+            else:
+                raise ValueError('Params to a DAG or a Task can be only of type airflow.models.param.Param')
+        return serialized_params
+
+    @classmethod
+    def _deserialize_params_dict(cls, encoded_params: Dict) -> ParamsDict:
+        """Deserialize a DAG's Params dict"""
+        op_params = {}
+        for k, v in encoded_params.items():
+            if isinstance(v, dict) and "__class" in v:
+                op_params[k] = cls._deserialize_param(v)
+            else:
+                # Old style params, convert it
+                op_params[k] = Param(v)
+
+        return ParamsDict(op_params)
+
 
 class DependencyDetector:
     """Detects dependencies between DAGs."""
@@ -385,6 +517,7 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
         # Move class attributes into object attributes.
         self.ui_color = BaseOperator.ui_color
         self.ui_fgcolor = BaseOperator.ui_fgcolor
+        self.template_ext = BaseOperator.template_ext
         self.template_fields = BaseOperator.template_fields
         self.operator_extra_links = BaseOperator.operator_extra_links
 
@@ -428,7 +561,10 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
                     )
 
                 deps.append(f'{module_name}.{klass.__name__}')
-            serialize_op['deps'] = deps
+            # deps needs to be sorted here, because op.deps is a set, which is unstable when traversing,
+            # and the same call may get different results.
+            # When calling json.dumps(self.data, sort_keys=True) to generate dag_hash, misjudgment will occur
+            serialize_op['deps'] = sorted(deps)
 
         # Store all template_fields as they are if there are JSON Serializable
         # If not, store them as strings
@@ -438,6 +574,9 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
                 if not cls._is_excluded(value, template_field, op):
                     serialize_op[template_field] = serialize_template_field(value)
 
+        if op.params:
+            serialize_op['params'] = cls._serialize_params_dict(op.params)
+
         return serialize_op
 
     @classmethod
@@ -453,7 +592,7 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
         op_extra_links_from_plugin = {}
 
         # We don't want to load Extra Operator links in Scheduler
-        if cls._load_operator_extra_links:  # pylint: disable=too-many-nested-blocks
+        if cls._load_operator_extra_links:
             from airflow import plugins_manager
 
             plugins_manager.initialize_extra_operators_links_plugins()
@@ -502,6 +641,8 @@ class SerializedBaseOperator(BaseOperator, BaseSerialization):
 
             elif k == "deps":
                 v = cls._deserialize_deps(v)
+            elif k == "params":
+                v = cls._deserialize_params_dict(v)
             elif k in cls._decorated_fields or k not in op.get_serialized_fields():
                 v = cls._deserialize(v)
             # else use v as it is
@@ -655,9 +796,9 @@ class SerializedDAG(DAG, BaseSerialization):
     _decorated_fields = {'schedule_interval', 'default_args', '_access_control'}
 
     @staticmethod
-    def __get_constructor_defaults():  # pylint: disable=no-method-argument
+    def __get_constructor_defaults():
         param_to_attr = {
-            'concurrency': '_concurrency',
+            'max_active_tasks': '_max_active_tasks',
             'description': '_description',
             'default_view': '_default_view',
             'access_control': '_access_control',
@@ -679,6 +820,13 @@ class SerializedDAG(DAG, BaseSerialization):
         try:
             serialize_dag = cls.serialize_to_json(dag, cls._decorated_fields)
 
+            # If schedule_interval is backed by timetable, serialize only
+            # timetable; vice versa for a timetable backed by schedule_interval.
+            if dag.timetable.summary == dag.schedule_interval:
+                del serialize_dag["schedule_interval"]
+            else:
+                del serialize_dag["timetable"]
+
             serialize_dag["tasks"] = [cls._serialize(task) for _, task in dag.task_dict.items()]
             serialize_dag["dag_dependencies"] = [
                 vars(t)
@@ -689,6 +837,7 @@ class SerializedDAG(DAG, BaseSerialization):
 
             # Edge info in the JSON exactly matches our internal structure
             serialize_dag["edge_info"] = dag.edge_info
+            serialize_dag["params"] = cls._serialize_params_dict(dag.params)
 
             # has_on_*_callback are only stored if the value is True, as the default is False
             if dag.has_on_success_callback:
@@ -698,8 +847,8 @@ class SerializedDAG(DAG, BaseSerialization):
             return serialize_dag
         except SerializationError:
             raise
-        except Exception:
-            raise SerializationError(f'Failed to serialize dag {dag.dag_id!r}')
+        except Exception as e:
+            raise SerializationError(f'Failed to serialize DAG {dag.dag_id!r}: {e}')
 
     @classmethod
     def deserialize_dag(cls, encoded_dag: Dict[str, Any]) -> 'SerializedDAG':
@@ -710,28 +859,40 @@ class SerializedDAG(DAG, BaseSerialization):
             if k == "_downstream_task_ids":
                 v = set(v)
             elif k == "tasks":
-                # pylint: disable=protected-access
+
                 SerializedBaseOperator._load_operator_extra_links = cls._load_operator_extra_links
-                # pylint: enable=protected-access
+
                 v = {task["task_id"]: SerializedBaseOperator.deserialize_operator(task) for task in v}
                 k = "task_dict"
             elif k == "timezone":
                 v = cls._deserialize_timezone(v)
-            elif k in {"dagrun_timeout"}:
+            elif k == "dagrun_timeout":
                 v = cls._deserialize_timedelta(v)
             elif k.endswith("_date"):
                 v = cls._deserialize_datetime(v)
             elif k == "edge_info":
                 # Value structure matches exactly
                 pass
+            elif k == "timetable":
+                v = _decode_timetable(v)
             elif k in cls._decorated_fields:
                 v = cls._deserialize(v)
+            elif k == "params":
+                v = cls._deserialize_params_dict(v)
             # else use v as it is
 
             setattr(dag, k, v)
 
+        # A DAG is always serialized with only one of schedule_interval and
+        # timetable. This back-populates the other to ensure the two attributes
+        # line up correctly on the DAG instance.
+        if "timetable" in encoded_dag:
+            dag.schedule_interval = dag.timetable.summary
+        else:
+            dag.timetable = create_timetable(dag.schedule_interval, dag.timezone)
+
         # Set _task_group
-        # pylint: disable=protected-access
+
         if "_task_group" in encoded_dag:
             dag._task_group = SerializedTaskGroup.deserialize_task_group(  # type: ignore
                 encoded_dag["_task_group"], None, dag.task_dict
@@ -742,7 +903,6 @@ class SerializedDAG(DAG, BaseSerialization):
             dag._task_group = TaskGroup.create_root(dag)
             for task in dag.tasks:
                 dag.task_group.add(task)
-        # pylint: enable=protected-access
 
         # Set has_on_*_callbacks to True if they exist in Serialized blob as False is the default
         if "has_on_success_callback" in encoded_dag:
@@ -754,7 +914,6 @@ class SerializedDAG(DAG, BaseSerialization):
         for k in keys_to_set_none:
             setattr(dag, k, None)
 
-        setattr(dag, 'full_filepath', dag.fileloc)
         for task in dag.task_dict.values():
             task.dag = dag
             serializable_task: BaseOperator = task
@@ -769,7 +928,7 @@ class SerializedDAG(DAG, BaseSerialization):
 
             for task_id in serializable_task.downstream_task_ids:
                 # Bypass set_upstream etc here - it does more than we want
-                # noqa: E501 # pylint: disable=protected-access
+
                 dag.task_dict[task_id]._upstream_task_ids.add(serializable_task.task_id)
 
         return dag
@@ -801,8 +960,11 @@ class SerializedTaskGroup(TaskGroup, BaseSerialization):
         if not task_group:
             return None
 
+        # task_group.xxx_ids needs to be sorted here, because task_group.xxx_ids is a set,
+        # when converting set to list, the order is uncertain.
+        # When calling json.dumps(self.data, sort_keys=True) to generate dag_hash, misjudgment will occur
         serialize_group = {
-            "_group_id": task_group._group_id,  # pylint: disable=protected-access
+            "_group_id": task_group._group_id,
             "prefix_group_id": task_group.prefix_group_id,
             "tooltip": task_group.tooltip,
             "ui_color": task_group.ui_color,
@@ -813,10 +975,10 @@ class SerializedTaskGroup(TaskGroup, BaseSerialization):
                 else (DAT.TASK_GROUP, SerializedTaskGroup.serialize_task_group(child))
                 for label, child in task_group.children.items()
             },
-            "upstream_group_ids": cls._serialize(list(task_group.upstream_group_ids)),
-            "downstream_group_ids": cls._serialize(list(task_group.downstream_group_ids)),
-            "upstream_task_ids": cls._serialize(list(task_group.upstream_task_ids)),
-            "downstream_task_ids": cls._serialize(list(task_group.downstream_task_ids)),
+            "upstream_group_ids": cls._serialize(sorted(task_group.upstream_group_ids)),
+            "downstream_group_ids": cls._serialize(sorted(task_group.downstream_group_ids)),
+            "upstream_task_ids": cls._serialize(sorted(task_group.upstream_task_ids)),
+            "downstream_task_ids": cls._serialize(sorted(task_group.downstream_task_ids)),
         }
 
         return serialize_group
