@@ -16,7 +16,7 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-/* global $, moment, Airflow, window, localStorage, document */
+/* global $, moment, Airflow, window, localStorage, document, hostName, csrfToken */
 
 import {
   dateTimeAttrFormat,
@@ -27,7 +27,11 @@ import {
 
 window.isoDateToTimeEl = isoDateToTimeEl;
 
-// We pull moment in via a webpack entrypoint rather than import so that we don't put it in more than a single .js file. This "exports" it to be globally available.
+/*
+ We pull moment in via a webpack entrypoint rather than import
+ so that we don't put it in more than a single .js file.
+ This "exports" it to be globally available.
+*/
 window.moment = Airflow.moment;
 
 function displayTime() {
@@ -47,7 +51,7 @@ function changDisplayedTimezone(tz) {
   });
 }
 
-var el = document.createElement('span');
+const el = document.createElement('span');
 
 export function escapeHtml(text) {
   el.textContent = text;
@@ -57,33 +61,33 @@ export function escapeHtml(text) {
 window.escapeHtml = escapeHtml;
 
 export function convertSecsToHumanReadable(seconds) {
-  var oriSeconds = seconds;
-  var floatingPart = oriSeconds- Math.floor(oriSeconds);
+  const oriSeconds = seconds;
+  const floatingPart = oriSeconds - Math.floor(oriSeconds);
 
   seconds = Math.floor(seconds);
 
-  var secondsPerHour = 60 * 60;
-  var secondsPerMinute = 60;
+  const secondsPerHour = 60 * 60;
+  const secondsPerMinute = 60;
 
-  var hours = Math.floor(seconds / secondsPerHour);
-  seconds = seconds - hours * secondsPerHour;
+  const hours = Math.floor(seconds / secondsPerHour);
+  seconds -= hours * secondsPerHour;
 
-  var minutes = Math.floor(seconds / secondsPerMinute);
-  seconds = seconds - minutes * secondsPerMinute;
+  const minutes = Math.floor(seconds / secondsPerMinute);
+  seconds -= minutes * secondsPerMinute;
 
-  var readableFormat = '';
+  let readableFormat = '';
   if (hours > 0) {
-    readableFormat += hours + 'Hours ';
+    readableFormat += `${hours}Hours `;
   }
   if (minutes > 0) {
-    readableFormat += minutes + 'Min ';
+    readableFormat += `${minutes}Min `;
   }
   if (seconds + floatingPart > 0) {
     if (Math.floor(oriSeconds) === oriSeconds) {
-      readableFormat += seconds + 'Sec';
+      readableFormat += `${seconds}Sec`;
     } else {
       seconds += floatingPart;
-      readableFormat += seconds.toFixed(3) + 'Sec';
+      readableFormat += `${seconds.toFixed(3)}Sec`;
     }
   }
   return readableFormat;
@@ -91,13 +95,13 @@ export function convertSecsToHumanReadable(seconds) {
 window.convertSecsToHumanReadable = convertSecsToHumanReadable;
 
 function postAsForm(url, parameters) {
-  var form = $('<form></form>');
+  const form = $('<form></form>');
 
   form.attr('method', 'POST');
   form.attr('action', url);
 
-  $.each(parameters || {}, function(key, value) {
-    var field = $('<input></input>');
+  $.each(parameters || {}, (key, value) => {
+    const field = $('<input></input>');
 
     field.attr('type', 'hidden');
     field.attr('name', key);
@@ -106,7 +110,7 @@ function postAsForm(url, parameters) {
     form.append(field);
   });
 
-  var field = $('<input></input>');
+  const field = $('<input></input>');
 
   field.attr('type', 'hidden');
   field.attr('name', 'csrf_token');
@@ -130,7 +134,7 @@ function initializeUITimezone() {
 
   function setManualTimezone(tz) {
     localStorage.setItem('chosen-timezone', tz);
-    if (tz == local && tz == Airflow.serverTimezone) {
+    if (tz === local && tz === Airflow.serverTimezone) {
       $('#timezone-manual').hide();
       return;
     }
@@ -203,11 +207,11 @@ $(document).ready(() => {
   setInterval(displayTime, 1000);
 
   $.ajaxSetup({
-    beforeSend: function(xhr, settings) {
+    beforeSend(xhr, settings) {
       if (!/^(GET|HEAD|OPTIONS|TRACE)$/i.test(settings.type) && !this.crossDomain) {
-        xhr.setRequestHeader("X-CSRFToken", csrfToken);
+        xhr.setRequestHeader('X-CSRFToken', csrfToken);
       }
-    }
+    },
   });
 
   $.fn.datetimepicker.defaults.format = 'YYYY-MM-DD HH:mm:ssZ';
